@@ -10,14 +10,20 @@ import re
 import sys
 import time
 import traceback
-from pathlib import Path
-from typing import List
 
 import pendulum
 from colorama import Fore
+from colorama.ansi import AnsiFore
 from colorlog import ColoredFormatter, default_log_colors
+from pathlib import Path
+from typing import List, TYPE_CHECKING
 
-if sys.version_info[0] < 3:
+if TYPE_CHECKING:
+    pass
+
+PYTHON2 = sys.version_info[0] < 3
+
+if PYTHON2:
     pass
 else:
     # noinspection PyShadowingBuiltins
@@ -50,11 +56,12 @@ class InfoFilter(logging.Filter):
 
 
 class Log(object):
-    LOGGER_LEVELS_DICT = {'CRITICAL': logging.CRITICAL,
-                          'ERROR': logging.ERROR,
-                          'WARNING': logging.WARNING,
-                          'INFO': logging.INFO,
-                          'DEBUG': logging.DEBUG}
+    if PYTHON2:
+        # noinspection PyProtectedMember
+        LOGGER_LEVELS_DICT = {k: v for k, v in logging._levelNames.items() if not isinstance(k, int)}
+    else:
+        # noinspection PyProtectedMember
+        LOGGER_LEVELS_DICT = logging._nameToLevel
     LOGGER_LEVELS = Struct(**LOGGER_LEVELS_DICT)
     GLOBAL_LOG_LEVEL = logging.INFO
     LOGGER_MESSAGE_FORMAT = '%(asctime)s.%(msecs)03d %(lineno)3s:%(name)-22s %(levelname)-6s %(message)s'
@@ -62,11 +69,12 @@ class Log(object):
     LOGGER_COLORED_MESSAGE_FORMAT = '%(log_color)s%(message)s'
     LOGGER_DATE_FORMAT = '%H:%M:%S'
     MAX_LOG_FILES = 50
-    DEFAULT_LOGS_DIR = Path(__file__).parent.parent.parent.resolve() / 'logs'
+    # DEFAULT_LOGS_DIR = Path(__file__).parent.parent.parent.resolve() / 'logs'
+    DEFAULT_LOGS_DIR = 'logs'
     loggers = {}
     auto_added_handlers = []  # type: List[logging.Handler]
     log_session_filename = None
-    logs_dir = DEFAULT_LOGS_DIR
+    logs_dir = DEFAULT_LOGS_DIR  # type: Path
 
     @staticmethod
     def set_global_log_level(level):
@@ -92,16 +100,16 @@ class Log(object):
         Log.MAX_LOG_FILES = max_log_files or Log.MAX_LOG_FILES
         Log.logs_dir = logs_dir or Log.logs_dir or Log.DEFAULT_LOGS_DIR
 
-        self.dump_initial_data = dump_initial_data
-        self.log = logging.getLogger(self.name)
-        self.log.propagate = False  # this fixes a recursion if other modules also use logging
+        self._dump_initial_data = dump_initial_data
+        self.logger = logging.getLogger(self.name)
+        self.logger.propagate = False  # this fixes a recursion if other modules also use logging
 
-        self.debug = self.log.debug
-        self.info = self.log.info
-        self.warning = self.log.warning
-        self.error = self.log.error
-        self.critical = self.log.critical
-        self.exception = self.log.exception
+        self.debug = self.logger.debug
+        self.info = self.logger.info
+        self.warning = self.logger.warning
+        self.error = self.logger.error
+        self.critical = self.logger.critical
+        self.exception = self.logger.exception
 
         new_log_file = False
         if Log.logs_dir:
@@ -116,7 +124,7 @@ class Log(object):
                 except InvalidZoneinfoFile:
                     now = pendulum.now(pendulum.UTC)  # travis-ci precaution
                 Log.log_session_filename = "%s.log" % now.strftime('%Y-%m-%d_%H-%M-%S')
-                self.clean_logs_folder()
+                self._clean_logs_folder()
                 new_log_file = True
 
         formatter = logging.Formatter(Log.LOGGER_MESSAGE_FORMAT, datefmt=Log.LOGGER_DATE_FORMAT_FULL)
@@ -124,24 +132,24 @@ class Log(object):
         color_formatter = ColoredFormatter(fmt=Log.LOGGER_COLORED_MESSAGE_FORMAT, datefmt=Log.LOGGER_DATE_FORMAT,
                                            reset=True, log_colors=default_log_colors)
 
-        self.stdout_handler = logging.StreamHandler(sys.stdout)
-        self.stdout_handler.addFilter(InfoFilter())
-        self.log.addHandler(self.stdout_handler)
-        self.stdout_handler.setFormatter(color_formatter)
+        self._stdout_handler = logging.StreamHandler(sys.stdout)
+        self._stdout_handler.addFilter(InfoFilter())
+        self.logger.addHandler(self._stdout_handler)
+        self._stdout_handler.setFormatter(color_formatter)
 
-        self.stderr_handler = logging.StreamHandler(sys.stderr)
-        self.log.addHandler(self.stderr_handler)
-        self.stderr_handler.setFormatter(color_formatter)
+        self._stderr_handler = logging.StreamHandler(sys.stderr)
+        self.logger.addHandler(self._stderr_handler)
+        self._stderr_handler.setFormatter(color_formatter)
 
         if Log.logs_dir:
             self.log_file_full_path = Log.logs_dir / Log.log_session_filename
-            self.filehandler = logging.FileHandler(str(self.log_file_full_path))
-            self.filehandler.setFormatter(formatter)
-            self.log.addHandler(self.filehandler)
+            self._filehandler = logging.FileHandler(str(self.log_file_full_path), encoding='UTF-8')
+            self._filehandler.setFormatter(formatter)
+            self.logger.addHandler(self._filehandler)
 
         self.level = level
         Log.loggers[self.name] = self
-        if Log.logs_dir and self.dump_initial_data and new_log_file:
+        if Log.logs_dir and self._dump_initial_data and new_log_file:
             try:
                 _env_dump_str = "Python {sysver} @ {platf}\n{user}@{comp} @ {winver}".format(
                     sysver=sys.version, platf=platform.architecture(), winver=sys.getwindowsversion(),
@@ -176,41 +184,64 @@ class Log(object):
     @classmethod
     def get_logger(cls, name=None, level=None, logs_dir=None, dump_initial_data=False, max_log_files=None,
                    message_format=None, date_format_full=None, date_format=None):
+        """
+        Main instantiating method for the class. Use it to instantiate global logger.
+
+        :param name: a unique logger name that is re-/used if already exists, defaults to the function path.
+        :type name: str
+        :param level: Logging level for the current instance.
+        :type level: int
+        :param logs_dir: Path where the .log files would be created, if provided.
+        :type logs_dir: Path or str
+        :param dump_initial_data: Whether to fill the newly created .log file with initial user data.
+        :type dump_initial_data: bool
+        :param max_log_files: Maximum .log files to store.
+        :type max_log_files: int
+        :param message_format: Logging message format.
+        :type message_format: str
+        :param date_format_full: Logging full date format.
+        :type date_format_full: str
+        :param date_format: Logging on-screen date format.
+        :type date_format: str
+        :return: :class:`Log` instance to work with.
+        :rtype: :class:`Log`
+        """
         name = name or get_prev_function_name()
         output = Log.loggers.get(name) or cls(name, level=level, logs_dir=logs_dir, dump_initial_data=dump_initial_data,
                                               max_log_files=max_log_files, message_format=message_format,
                                               date_format_full=date_format_full, date_format=date_format)
-        Log.add_autoadded_handlers()
+        Log._add_autoadded_handlers()
         return output
 
     @staticmethod
-    def add_autoadded_handlers():
+    def _add_autoadded_handlers():
         for handler in Log.auto_added_handlers:
             for logger_name, logger in Log.loggers.items():
-                if handler not in logger.log.handlers:
-                    logger.log.addHandler(handler)
+                if handler not in logger.logger.handlers:
+                    logger.logger.addHandler(handler)
 
     @property
     def level(self):
         if hasattr(self, 'stdout_handler'):
-            return self.stdout_handler.level
+            return self._stdout_handler.level
 
     @level.setter
     def level(self, value):
         Log.GLOBAL_LOG_LEVEL = value
-        self.log.setLevel(logging.DEBUG)
+        self.logger.setLevel(logging.DEBUG)
         if Log.logs_dir:
-            self.filehandler.setLevel(logging.DEBUG)
-        self.stdout_handler.setLevel(value)
-        self.stderr_handler.setLevel(logging.WARNING)
+            self._filehandler.setLevel(logging.DEBUG)
+        self._stdout_handler.setLevel(value)
+        self._stderr_handler.setLevel(logging.WARNING)
 
     @property
-    def log_files(self):
+    def _log_files(self):
         output = sorted(list(Log.logs_dir.glob('*.log')), key=lambda f: f.stat().st_ctime,
                         reverse=True)
         return output
 
-    def clean_logs_folder(self):
+    @staticmethod
+    def _clean_logs_folder():
         log_files = sorted(list(Log.logs_dir.glob('*.log')), key=lambda f: f.stat().st_ctime,
                            reverse=True)
         if len(log_files) > Log.MAX_LOG_FILES:
@@ -229,6 +260,17 @@ class Log(object):
         return self.printer(color='yellow', *message, **kwargs)
 
     def printer(self, *message, **kwargs):
+        """
+
+        :param message:
+        :type message: str or list of str
+        :param end:
+        :type end: str
+        :param color:
+        :type color: AnsiFore
+        :param clear:
+        :type clear: bool
+        """
         default_end = '\n'
         end = kwargs.get(str('end'), None)
         color = kwargs.get(str('color'))
@@ -238,15 +280,13 @@ class Log(object):
             timestamp = '' if end == '' else '%s ' % time.strftime(str("%H:%M:%S"))
 
             if Log.logs_dir:
-                if sys.version_info[0] < 3:
-                    msg = unicode(msg)  # noqa
                 _timestamped_message = '%s%s' % (timestamp, msg)
                 _cleared_timestamped_message = clear_message(_timestamped_message)
-                self.file_printer(_cleared_timestamped_message)
+                self._file_printer(_cleared_timestamped_message)
 
             # todo: emit to custom handlers
             for handler in Log.auto_added_handlers:
-                handler.emit(msg.encode('UTF-8'))
+                handler.emit(msg)
 
             _cleared_message = msg
             if clear is True and isinstance(msg, (str, buffer)):
@@ -254,19 +294,23 @@ class Log(object):
 
             _colored_msg = _cleared_message
             if color:
-                color = getattr(Fore, color.upper(), Fore.GREEN)
+                if not isinstance(color, AnsiFore):
+                    color = getattr(Fore, color.upper(), Fore.GREEN)
                 _colored_msg = '%s%s%s' % (color, _cleared_message, Fore.RESET)
 
             print(_colored_msg, end=print_end)
 
-    def file_printer(self, message):
+    def _file_printer(self, msg):
         if not Log.logs_dir:
             return
 
-        if not message.endswith('\n'):
-            message += '\n'
-        self.filehandler.stream.write(str(message.encode('UTF-8')))
-        self.filehandler.flush()
+        if not msg.endswith('\n'):
+            msg += '\n'
+
+        # if not PYTHON2 and isinstance(msg, str):
+        #     msg = msg.encode('UTF-8')
+        self._filehandler.stream.write(msg)
+        self._filehandler.flush()
 
     def trace(self):
         frame = inspect.currentframe().f_back
@@ -278,6 +322,37 @@ class Log(object):
         _params = [(i, values[i]) for i in args if 'self' not in i]
         self.debug("%s.%s.%s%s" % (file_dir, file_name, func_name, _params))
 
+    # @staticmethod
+    # def _string_magic(msg):
+    #     # if not PYTHON2 and isinstance(msg, str):
+    #     #     return msg.encode('UTF-8')
+    #
+    #     return msg
+    #
+    # def debug(self, msg, *args, **kwargs):
+    #     msg = self._string_magic(msg)
+    #     return self.logger.debug(msg, *args, **kwargs)
+    #
+    # def info(self, msg, *args, **kwargs):
+    #     return self.logger.info(self._string_magic(msg), *args, **kwargs)
+    #
+    # def warning(self, msg, *args, **kwargs):
+    #     return self.logger.warning(self._string_magic(msg), *args, **kwargs)
+    #
+    # def error(self, msg, *args, **kwargs):
+    #     return self.logger.error(self._string_magic(msg), *args, **kwargs)
+    #
+    # def critical(self, msg, *args, **kwargs):
+    #     return self.logger.critical(self._string_magic(msg), *args, **kwargs)
+    #
+    # def exception(self, msg, *args, exc_info=True, **kwargs):
+    #     return self.logger.info(self._string_magic(msg), *args, exc_info=exc_info, **kwargs)
+
+
+def __func():
+    log.trace()
+    log.debug('func called')
+
 
 if __name__ == '__main__':
     log = Log.get_logger()
@@ -287,4 +362,5 @@ if __name__ == '__main__':
     log.printer('test filehandler message абракадабра')
     log.warning('test warning абракадабра')
     log.green('test green абракадабра')
-    pass
+    __func()
+    print("")
